"use strict";
/**
 * Copyright 2023 actions-toolkit authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Buildx = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const core = __importStar(require("@actions/core"));
const semver = __importStar(require("semver"));
const docker_1 = require("../docker/docker");
const exec_1 = require("../exec");
class Buildx {
    _version;
    _versionOnce;
    _standalone;
    static containerNamePrefix = 'buildx_buildkit_';
    constructor(opts) {
        this._standalone = opts?.standalone;
        this._version = '';
        this._versionOnce = false;
    }
    static get configDir() {
        return process.env.BUILDX_CONFIG || path_1.default.join(docker_1.Docker.configDir, 'buildx');
    }
    static get certsDir() {
        return path_1.default.join(Buildx.configDir, 'certs');
    }
    async isStandalone() {
        const standalone = this._standalone ?? !(await docker_1.Docker.isAvailable());
        core.debug(`Buildx.isStandalone: ${standalone}`);
        return standalone;
    }
    async getCommand(args) {
        const standalone = await this.isStandalone();
        return {
            command: standalone ? 'buildx' : 'docker',
            args: standalone ? args : ['buildx', ...args]
        };
    }
    async isAvailable() {
        const cmd = await this.getCommand([]);
        const ok = await exec_1.Exec.getExecOutput(cmd.command, cmd.args, {
            ignoreReturnCode: true,
            silent: true
        })
            .then(res => {
            if (res.stderr.length > 0 && res.exitCode != 0) {
                core.debug(`Buildx.isAvailable cmd err: ${res.stderr.trim()}`);
                return false;
            }
            return res.exitCode == 0;
        })
            .catch(error => {
            core.debug(`Buildx.isAvailable error: ${error}`);
            return false;
        });
        core.debug(`Buildx.isAvailable: ${ok}`);
        return ok;
    }
    async printInspect(name) {
        const cmd = await this.getCommand(['inspect', name]);
        await exec_1.Exec.exec(cmd.command, cmd.args, {
            failOnStdErr: false
        });
    }
    async version() {
        if (this._versionOnce) {
            return this._version;
        }
        this._versionOnce = true;
        const cmd = await this.getCommand(['version']);
        this._version = await exec_1.Exec.getExecOutput(cmd.command, cmd.args, {
            ignoreReturnCode: true,
            silent: true
        }).then(res => {
            if (res.stderr.length > 0 && res.exitCode != 0) {
                throw new Error(res.stderr.trim());
            }
            return Buildx.parseVersion(res.stdout.trim());
        });
        return this._version;
    }
    async printVersion() {
        const cmd = await this.getCommand(['version']);
        await exec_1.Exec.exec(cmd.command, cmd.args, {
            failOnStdErr: false
        });
    }
    static parseVersion(stdout) {
        const matches = /\sv?([0-9a-f]{7}|[0-9.]+)/.exec(stdout);
        if (!matches) {
            throw new Error(`Cannot parse buildx version`);
        }
        return matches[1];
    }
    async versionSatisfies(range, version) {
        const ver = version ?? (await this.version());
        if (!ver) {
            core.debug(`Buildx.versionSatisfies false: undefined version`);
            return false;
        }
        const res = semver.satisfies(ver, range) || /^[0-9a-f]{7}$/.exec(ver) !== null;
        core.debug(`Buildx.versionSatisfies ${ver} statisfies ${range}: ${res}`);
        return res;
    }
    static resolveCertsDriverOpts(driver, endpoint, cert) {
        let url;
        try {
            url = new URL(endpoint);
        }
        catch (e) {
            return [];
        }
        if (url.protocol != 'tcp:') {
            return [];
        }
        const driverOpts = [];
        if (Object.keys(cert).length == 0) {
            return driverOpts;
        }
        let host = url.hostname;
        if (url.port.length > 0) {
            host += `-${url.port}`;
        }
        if (cert.cacert !== undefined) {
            const cacertpath = path_1.default.join(Buildx.certsDir, `cacert_${host}.pem`);
            fs_1.default.writeFileSync(cacertpath, cert.cacert);
            driverOpts.push(`cacert=${cacertpath}`);
        }
        if (cert.cert !== undefined) {
            const certpath = path_1.default.join(Buildx.certsDir, `cert_${host}.pem`);
            fs_1.default.writeFileSync(certpath, cert.cert);
            driverOpts.push(`cert=${certpath}`);
        }
        if (cert.key !== undefined) {
            const keypath = path_1.default.join(Buildx.certsDir, `key_${host}.pem`);
            fs_1.default.writeFileSync(keypath, cert.key);
            driverOpts.push(`key=${keypath}`);
        }
        if (driver != 'remote') {
            return [];
        }
        return driverOpts;
    }
}
exports.Buildx = Buildx;
//# sourceMappingURL=buildx.js.map